from math import sqrt,hypot,atan2,pi,radians,sin,cos

class Vector2D:
    def __init__(self,x,y):
        self.x,self.y = x,y
    def __add__(self,rhs):
        return Vector2D(self.x + rhs.x, self.y + rhs.y)
    def __sub__(self,rhs):
        return Vector2D(self.x - rhs.x, self.y - rhs.y)
    def __mul__(self,rhs):
        return Vector2D(self.x * rhs, self.y * rhs)
    def __div__(self,rhs):
        return Vector2D(self.x / rhs, self.y / rhs)
    def __iadd__(self,rhs):
        self.x += rhs.x
        self.y += rhs.y
        return self
    def __isub__(self,rhs):
        self.x -= rhs.x
        self.y -= rhs.y
        return self
    def __imul__(self,rhs):
        self.x *= rhs
        self.y *= rhs
        return self
    def __idiv__(self,rhs):
        self.x /= rhs
        self.y /= rhs
        return self
    def __neg__(self):
        return Vector2D(-self.x,-self.y)
    def __str__(self):
        return "Vector2D(%d,%d)" % (self.x,self.y)
    
def Length(vec):
    return hypot(vec.x,vec.y)

def Normalized(vec):
    if vec.x == 0 and vec.y == 0:
        return Vector2D(0.0,0.0)
    return vec / Length(vec)

def Distance(v1,v2):
    return hypot(v1.x-v2.x, v1.y-v2.y)

def DistanceSqrt(v1,v2):
    return (v1.x-v2.x)*(v1.x-v2.x) + (v1.y-v2.y)*(v1.y-v2.y)

rad = 180/pi

def LookAt(v1,v2):
    """V2 looks at v1. Returns rotation."""
    return atan2(v1.x - v2.x, v1.y - v2.y) * rad

def MoveLocal(vec,rot,dist):
    angleRadians = -radians(rot)
    vec.x += -sin(angleRadians) * dist
    vec.y += cos(angleRadians) * dist

def Rotate(vec, rot):
    angleRadians = -radians(rot)
    c = cos(angleRadians)
    s = sin(angleRadians)
    x = vec.x*c - vec.y*s
    y = vec.x*s + vec.y*c
    vec.x = x
    vec.y = y
    