from Utils import stackless,pyglet,randrange,Distance

class World:
    def __init__(self,window,width,height):
        self.window = window
        self.keys = pyglet.window.key.KeyStateHandler()
        self.window.push_handlers(self.keys)
        
        self.bullets = []
        self.ships = []
        self.mainShip = None
        self.turrets = []
        self.bonuses = []
        self.planets = []
        self.asteroids = []
        self.soundQueue = []
        
        self.width,self.height = width,height
        
        self.stars = []
        self.starsBatch = pyglet.graphics.Batch()
        for i in range(randrange(self.width >> 2,self.width << 1)):
            star = pyglet.sprite.Sprite(pyglet.resource.image("star.png"),randrange(0,self.width),randrange(0,self.height),batch = self.starsBatch)
            self.stars.append(star)
            
    def addBullet(self,bullet):
        bullet.world = self
        self.bullets.append(bullet)
    
    def addShip(self,ship,controller):
        ship.world = self
        ship.controller = controller
        self.ships.append(ship)
    
    def addTurret(self,turret):
        turret.world = self
        self.turrets.append(turret)
    
    def addBonus(self,bonus):
        bonus.world = self
        self.bonuses.append(bonus)
        
    def addPlanet(self,planet):
        planet.world = self
        self.planets.append(planet)
        
    def addAsteroid(self,asteroid):
        asteroid.world = self
        self.asteroids.append(asteroid)
        
    def draw(self):
        self.starsBatch.draw()
        
        for object in self.planets + self.bonuses + self.bullets + self.turrets + self.ships + self.asteroids:
            if Distance(object.pos,self.mainShip.pos) <= 600:
                object.draw()
    
    def update(self,delta):
        self.delta = delta
        stackless.schedule()
        
        for bullet in self.bullets:
            bullet.update()
            if bullet.life <= 0.0:
                self.bullets.remove(bullet)
        
        for planet in self.planets:
            planet.update()
        
        for bonus in self.bonuses:
            bonus.update()
            if bonus.life <= 0.0:
                self.bonuses.remove(bonus)
        
        for asteroid in self.asteroids:
            print "Update!"
            asteroid.update()
            if asteroid.life <= 0.0:
                self.asteroids.remove(asteroid)
                
        for ship in self.ships:
            if ship.life <= 0.0:
                self.ships.remove(ship)
                
        for turret in self.turrets:
            if turret.life <= 0.0:
                self.turrets.remove(turret)
                
    def playSounds(self):
        player = pyglet.media.Player()
        for sound in self.soundQueue:
            dist = Distance(sound[1],self.mainShip.pos)
            if dist <= 800.0:
                snd = pyglet.resource.media(sound[0] + '.wav')
                player.volume = (800.0-dist)/800.0
                player.queue(snd)
                self.soundQueue.pop()
        player.play()
                